<?php

namespace SD\SmsApiSdk;

class ApiClient {
	private $api_id; 
	private $api_key;
    private $api_http_status_code;
    private $api_http_response;
    
	public $format= 'json';
    
    private $base_api_url = 'https://sms-api.syscomdigital.ro/';
    
    private $api_send_path = 'send/';
    
	
	public function __construct($api_id,$api_key){
		$this->api_id = $api_id;
		$this->api_key = $api_key;
	}

    /**
     * @param $phone - phone number
     * @param $text - SMS message
     * @param $country - contry ISO2 code
     * @param int $nrofc - number of concatenated messages
     * @param null $sendon
     * @param string $sender - alphanumeric sender, if available
     * @param string $delivery_url - delivery report url report feedback
     * @param string $response_format - response format (json,xml)
     * @param int $dlrmask - delivery report mask
     * @param string $my_sms_id (optional) - unique sms id, from API caller, in order to check and avoid duplicates
     * @return ApiClientResp|null
     * @throws ApiClientException
     */
	function send_sms($phone, $text, $country, $nrofc = 1, $sendon = NULL, $sender = NULL, $delivery_url = NULL, $response_format= 'json', $dlrmask = NULL, $my_sms_id = NULL){
		    
		$phone = $this->urlencode_rfc3986($phone);
		$text = $this->urlencode_rfc3986($text);
		$c = $this->urlencode_rfc3986($country);
        
        $date = new \DateTime('now',new \DateTimeZone('Europe/Bucharest'));

		$d = $this->urlencode_rfc3986($date->format('Y-m-d H:i:s'));
		$signature = $this->urlencode_rfc3986($this->_generate_hash($this->api_key,array($this->api_id,$phone,$c,$d)));
		$concat = $this->urlencode_rfc3986($nrofc);
        
        if($response_format !== NULL && strtolower($response_format) === 'json'){
            $this->format = $response_format;
        }
        
		
		$data = "accessid={$this->api_id}&phone={$phone}&text={$text}&c={$c}&d={$d}&signature={$signature}&nrofc={$concat}&format={$this->format}";
		if($sendon){
			$data.= '&sendat=' .$this->urlencode_rfc3986(date('Y-m-d H:i',$sendon));
		}
		if($sender){
			$data.= '&sender=' .$this->urlencode_rfc3986($sender);
		}
		if($delivery_url){
			$data.= '&delivery_url=' .$this->urlencode_rfc3986($delivery_url);
		}
        if($dlrmask){
            $data.= '&dlrmask=' .$this->urlencode_rfc3986($dlrmask);
        }

        if($my_sms_id){
            $data.= '&my_sms_id=' .$this->urlencode_rfc3986($my_sms_id);
        }
		
		try{
			$response_data = $this->_call_api($this->base_api_url.$this->api_send_path,$data);
		}catch(ApiClientException $e){
			throw $e;	
		}
        
		return $this->_get_api_response($response_data);
	}

    private function urlencode_rfc3986($str){
        //php >= 5.3.0
        return rawurlencode($str);
    }

    private function _generate_hash($key,$data=array()){
        $message = implode('&', $data);
        return base64_encode(hash_hmac('sha1',$message,$key,true));
    }
    
    
    private function _call_api($apiurl,$data){
        
        $ch = curl_init($apiurl) ;
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
        curl_setopt($ch, CURLOPT_HEADER, 0); 
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1) ;
        
        $this->api_http_response = curl_exec ($ch);         
        $this->api_http_status_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);        
        
        if((int)$this->api_http_status_code === 403){
            throw new ApiClientException($this->api_http_status_code ,$this->api_http_response,new \Exception('Access Forbidden!'));
        }else
        if((int)$this->api_http_status_code !== 200){
            throw new ApiClientException($this->api_http_status_code ,$this->api_http_response,new \Exception('Request Failed!'));
        }
        
        return $this->api_http_response;
    }
    
    
	
	private function _get_api_response($response_data){
		$r = null; 
		$xml = null;
		$json = null;
		$d = null;
		if($this->format=="xml"){
	       libxml_use_internal_errors(true);
			$xml = simplexml_load_string($response_data, null, LIBXML_NOCDATA);
			if($xml === FALSE) throw new ApiClientException($this->api_http_status_code,$this->api_http_response,new \Exception('XML parse error'));
			$r = new ApiClientResp($xml->sms->attributes()->status,$xml->sms->attributes()->id,$xml->sms->statusResponse);    
		}
		elseif($this->format=="json"){
		        
		   $json = json_decode($response_data,true);
         
           if(json_last_error()>0)  throw new ApiClientException($this->api_http_status_code,$this->api_http_response,new \Exception('JSON parse error: ' .json_last_error()));

	       $r = new ApiClientResp($json["sms"]["status"],$json["sms"]["id"],$json["sms"]["statusResponse"]);
  			
		}
		return $r;
	}
	
	
	
}




 
 
